@@ -41,10 +41,18 @@ class Transformation(metaclass=ABCMeta):
     # no further changes will be committed.
     _return_type = 'pandas'
 
-    # A tuple indicating which arguments give the names of variables that must
-    # all be aligned with one another (i.e., onsets and durations match
-    # perfectly) before processing. Defaults to None.
-    _align = None
+    # Indicates if variables must be aligned with one another
+    # (i.e., onsets and durations match perfectly) before processing.
+    # If True, will require all variables to be aligned (throw exception).
+    # If 'force', will auto-align variables by densification.
+    # Defaults to None.
+    _aligned_required = None
+
+    # (Optional) If alignment is required,sets which variables must be aligned.
+    # If None, all variables must be aligned. It can also be a a string
+    # indicating which column contains the names of columns to be aligned,
+    # or a tuple which explicitly names the columns to be aligned.
+    _aligned_variables = None
 
     # Boolean indicating whether the Transformation should be applied to each
     # variable in the input list in turn. When True (default), Transformation
@@ -311,7 +319,7 @@ class Transformation(metaclass=ABCMeta):
     def _postprocess(self, col):
         return col
 
-    def _align_variables(self, variables, force=True):
+    def _align_variables(self, variables):
         """Checks whether the specified variables have aligned indexes. This
         implies either that all variables are dense, or that all variables are
         sparse and have exactly the same onsets and durations. If variables are
@@ -319,7 +327,7 @@ class Transformation(metaclass=ABCMeta):
         format in order to ensure alignment.
         """
 
-        if self._align is None or self._align == 'none':
+        if self._aligned_required is None or self._aligned_required == 'none':
             return
 
         def _align(variables):
@@ -328,15 +336,8 @@ class Transformation(metaclass=ABCMeta):
                       if isinstance(c, SparseRunVariable)]
             if len(sparse) < len(variables):
                 if sparse:
-                    sparse_names = [s.name for s in sparse]
                     msg = ("Found a mix of dense and sparse variables. May "
                            "cause problems for some transformations.")
-                    if force:
-                        msg += (" Sparse variables %s were converted to dense "
-                                "form to ensure proper alignment." %
-                                sparse_names)
-                        sr = self.collection.sampling_rate
-                        sparse = [s.to_dense(sr) for s in sparse]
                     warnings.warn(msg)
             # If all are sparse, durations, onsets, and index must match
             # perfectly for all
@@ -351,28 +352,37 @@ class Transformation(metaclass=ABCMeta):
                 fc = get_col_data(variables[0])
                 if not all([compare_variables(fc, get_col_data(c))
                             for c in variables[1:]]):
-                    msg = "Misaligned sparse variables found."
-                    if force:
-                        msg += (" Forcing all sparse variables to dense in "
-                                "order to ensure proper alignment.")
+                    if self._aligned_required == 'force_dense':
+                        msg = ("Forcing all sparse variables to dense in "
+                               "order to ensure proper alignment.")
                         sr = self.collection.sampling_rate
                         variables = [c.to_dense(sr) for c in variables]
-                    warnings.warn(msg)
+                        warnings.warn(msg)
+                    else:
+                        raise ValueError(
+                            "Misaligned sparse variables found."
+                            "To force variables into alignment by densifying,"
+                            "set dense=True in the Transformation arguments"
+                            )
 
-        align_variables = [listify(self.kwargs[v])
-                           for v in listify(self._align) if v in self.kwargs]
-        align_variables = list(itertools.chain(*align_variables))
-        align_variables = [self.collection[c] for c in align_variables if c]
+        _aligned_variables = True if not self._aligned_variables \
+            else self._aligned_variables
+        _aligned_variables = [listify(self.kwargs[v])
+                              for v in listify(_aligned_variables)
+                              if v in self.kwargs]
+        _aligned_variables = list(itertools.chain(*_aligned_variables))
+        _aligned_variables = [
+            self.collection[c] for c in _aligned_variables if c]
 
-        if align_variables and self._loopable:
+        if _aligned_variables and self._loopable:
             for c in variables:
                 # TODO: should clone all variables in align_variables before
                 # alignment to prevent conversion to dense in any given
                 # iteration having side effects. This could be an issue if,
                 # e.g., some vars in 'variables' are dense and some are sparse.
-                _align([c] + align_variables)
+                _align([c] + _aligned_variables)
         else:
-            _align(listify(variables) + align_variables)
+            _align(listify(variables) + _aligned_variables)
 
 
 class TransformerManager(object):
