@@ -3,14 +3,15 @@ BIDSLayout class."""
 
 import os
 import re
-from os.path import (join, abspath, basename)
+from os.path import join, abspath, basename
+from pathlib import Path
 
 import numpy as np
 import pytest
 
 from bids.layout import BIDSLayout, Query
-from bids.layout.index import BIDSLayoutIndexer
 from bids.layout.models import Config
+from bids.layout.index import BIDSLayoutIndexer
 from bids.tests import get_test_data_path
 from bids.utils import natural_sort
 
@@ -29,20 +30,17 @@ def test_layout_init(layout_7t_trt):
 @pytest.mark.parametrize(
     'index_metadata,query,result',
     [
-        (True, None, 3.0),
-        (False, None, None),
-        (False, {}, 3.0),
-        (False, {'task': 'rest'}, 3.0),
-        (False, {'task': 'rest', 'extension': ['.nii.gz']}, 3.0),
-        (False, {'task': 'rest', 'extension': '.nii.gz'}, 3.0),
-        (False, {'task': 'rest', 'extension': ['.nii.gz', '.json'], 'return_type': 'file'}, 3.0),
+        (True, {}, 3.0),
+        (False, {}, None),
+        (True, {}, 3.0),
+        (True, {'task': 'rest'}, 3.0),
+        (True, {'task': 'rest', 'extension': ['.nii.gz']}, 3.0),
+        (True, {'task': 'rest', 'extension': '.nii.gz'}, 3.0),
+        (True, {'task': 'rest', 'extension': ['.nii.gz', '.json'], 'return_type': 'file'}, 3.0),
     ])
 def test_index_metadata(index_metadata, query, result, mock_config):
     data_dir = join(get_test_data_path(), '7t_trt')
-    layout = BIDSLayout(data_dir, index_metadata=index_metadata)
-    if not index_metadata and query is not None:
-        indexer = BIDSLayoutIndexer(layout)
-        indexer.add_metadata(**query)
+    layout = BIDSLayout(data_dir, index_metadata=index_metadata, **query)
     sample_file = layout.get(task='rest', extension='.nii.gz',
                              acquisition='fullbrain')[0]
     metadata = sample_file.get_metadata()
@@ -188,23 +186,23 @@ def test_get_metadata_error(layout_7t_trt):
     result = layout_7t_trt.files[path].get_metadata()
     with pytest.raises(KeyError) as err:
         result['Missing']
-    assert "Metadata term 'Missing' unavailable for file {}".format(path) in str(err)
+    assert "Metadata term 'Missing' unavailable for file {}".format(path) in str(err.value)
 
     result = layout_7t_trt.get_metadata(path)
     with pytest.raises(KeyError) as err:
         result['Missing']
-    assert "Metadata term 'Missing' unavailable for file {}".format(path) in str(err)
+    assert "Metadata term 'Missing' unavailable for file {}".format(path) in str(err.value)
 
 
 def test_get_with_bad_target(layout_7t_trt):
     with pytest.raises(TargetError) as exc:
         layout_7t_trt.get(target='unicorn')
-        msg = exc.value.message
-        assert 'subject' in msg and 'reconstruction' in msg and 'proc' in msg
+    msg = str(exc.value)
+    assert 'subject' in msg and 'reconstruction' in msg and 'proc' in msg
     with pytest.raises(TargetError) as exc:
         layout_7t_trt.get(target='sub')
-        msg = exc.value.message
-        assert 'subject' in msg and 'reconstruction' not in msg
+    msg = str(exc.value)
+    assert 'subject' in msg and 'reconstruction' not in msg
 
 
 def test_get_bvals_bvecs(layout_ds005):
@@ -269,17 +267,6 @@ def test_get_return_type_dir(layout_7t_trt, layout_7t_trt_relpath):
     assert target == res2
 
 
-def test_ensure_non_detaching_sessions(layout_7t_trt, layout_7t_trt_relpath):
-    """ Ensure that sessions don't detach unexpectly during queries."""
-    query = dict(target='subject', return_type='dir')
-    res_relpath = layout_7t_trt_relpath.get(**query)
-    target_relpath = ["sub-{:02d}".format(i) for i in range(1, 11)]
-    assert target_relpath == res_relpath
-    res_abspath = layout_7t_trt.get(**query)
-    # Second query would break without expire_on_commit=True
-    res_relpath = layout_7t_trt_relpath.get(**query)
-
-
 @pytest.mark.parametrize("acq", [None, Query.NONE])
 def test_get_val_none(layout_7t_trt, acq):
     t1w_files = layout_7t_trt.get(subject='01', session='1', suffix='T1w')
@@ -324,7 +311,8 @@ def test_ignore_files(layout_ds005):
     # overrides the default - but 'model/extras/' should still be ignored
     # because of the regex.
     ignore = [re.compile('xtra'), 'dummy']
-    layout2 = BIDSLayout(data_dir, validate=False, ignore=ignore)
+    indexer = BIDSLayoutIndexer(validate=False, ignore=ignore)
+    layout2 = BIDSLayout(data_dir, indexer=indexer)
     assert target1 in layout2.files
     assert target2 not in layout2.files
 
@@ -332,7 +320,8 @@ def test_ignore_files(layout_ds005):
 def test_force_index(layout_ds005):
     data_dir = join(get_test_data_path(), 'ds005')
     target = join(data_dir, 'models', 'ds-005_type-test_model.json')
-    model_layout = BIDSLayout(data_dir, validate=True, force_index=['models'])
+    indexer = BIDSLayoutIndexer(force_index=['models'])
+    model_layout = BIDSLayout(data_dir, validate=True, indexer=indexer)
     assert target not in layout_ds005.files
     assert target in model_layout.files
     assert 'all' not in model_layout.get_subjects()
@@ -456,11 +445,11 @@ def test_get_tr(layout_7t_trt):
     # Bad subject, should fail
     with pytest.raises(NoMatchError) as exc:
         layout_7t_trt.get_tr(subject="zzz")
-        assert exc.value.message.startswith("No functional images")
+    assert str(exc.value).startswith("No functional images")
     # There are multiple tasks with different TRs, so this should fail
     with pytest.raises(NoMatchError) as exc:
         layout_7t_trt.get_tr(subject=['01', '02'])
-        assert exc.value.message.startswith("Unique TR")
+    assert str(exc.value).startswith("Unique TR")
     # This should work
     tr = layout_7t_trt.get_tr(subject=['01', '02'], acquisition="fullbrain")
     assert tr == 3.0
@@ -526,6 +515,21 @@ def test_deriv_indexing():
     assert not layout.get(scope='nonexistent')
 
 
+def test_path_arguments():
+    data_dir = join(get_test_data_path(), 'ds005')
+    deriv_dir = join(data_dir, 'derivatives', 'events')
+
+    layout = BIDSLayout(Path(data_dir), derivatives=Path(deriv_dir))
+    assert layout.get(scope='derivatives')
+    assert layout.get(scope='events')
+    assert not layout.get(scope='nonexistent')
+
+    layout = BIDSLayout(Path(data_dir), derivatives=[Path(deriv_dir)])
+    assert layout.get(scope='derivatives')
+    assert layout.get(scope='events')
+    assert not layout.get(scope='nonexistent')
+
+
 def test_layout_in_scope(layout_ds005, layout_ds005_derivs):
     assert layout_ds005._in_scope(['all'])
     assert layout_ds005._in_scope('raw')
@@ -599,8 +603,8 @@ def test_indexing_tag_conflict():
     data_dir = join(get_test_data_path(), 'ds005_conflict')
     with pytest.raises(BIDSValidationError) as exc:
         layout = BIDSLayout(data_dir)
-        assert exc.value.message.startswith("Conflicting values found")
-        assert 'run' in exc.value.message
+    assert str(exc.value).startswith("Conflicting values found")
+    assert 'run' in str(exc.value)
 
 
 def test_get_with_wrong_dtypes(layout_7t_trt):
@@ -675,4 +679,5 @@ def test_load_layout(layout_synthetic_nodb, db_dir):
         sorted(reloaded.get(return_type='file'))
     cm1 = layout_synthetic_nodb.connection_manager
     cm2 = reloaded.connection_manager
-    assert cm1.init_args == cm2.init_args
+    for attr in ['root', 'absolute_paths', 'config', 'derivatives']:
+        assert getattr(cm1.layout_info, attr) == getattr(cm2.layout_info, attr)
