@@ -46,9 +46,13 @@ class BIDSVariable(metaclass=ABCMeta):
         result = deepcopy(self)
         if data is not None:
             if data.shape != self.values.shape:
-                raise ValueError("Replacement data has shape %s; must have "
-                                 "same shape as existing data %s." %
-                                 (data.shape, self.values.shape))
+                # If data can be re-shaped safely, do so
+                if data.squeeze().shape == self.values.squeeze().shape:
+                    data = data.values.reshape(self.values.shape)
+                else:
+                    raise ValueError("Replacement data has shape %s; must have"
+                                     " same shape as existing data %s." %
+                                     (data.shape, self.values.shape))
             result.values = pd.DataFrame(data)
 
         if kwargs:
